<?php
//	Copyright (c) 2009, 2019 Daniel Robert Bradley. All rights reserved.
//	This software is distributed under the terms of the GNU Lesser General Public License version 2.1
?>
<?php

class Security
{
	static function IsAccessDenied( $pageid )
	{
		$is_denied = false;
		$status    = "";

		if ( string_startsWith( $pageid, "dashboard" ) )
		{
			$file1 = CONTENT_PATH . "/_site/_RESTRICT_DASHBOARD_ACCESS.txt";
			$file2 = CONTENT_PATH . "/_site/_RESTRICT_DASHBOARD_ACCESS";

			error_log( "Checking access config: " . $file1 );

			if ( file_exists( $file1 ) || file_exists( $file2 ) )
			{
				if ( ! array_key_exists( "accessid", $_COOKIE ) )
				{
					$status    = "Missing";
					$is_denied = true;
				}
				else
				{
					$accessid = $_COOKIE["accessid"];

					if ( 0 == strlen( $accessid ) )
					{
						$status    = "Empty";
						$is_denied = true;
					}
					else
					{
						if ( $is_denied = self::IsInvalidAccessID( $accessid ) )
						{
							$status = "Invalid";
						}
					}
				}
			}
		}

		if ( $is_denied )
		{
			error_log( "ACCESS DENIED: " . $status );

			error_log( var_export( $_COOKIE, true ) );
		}

		return $is_denied;
	}

	static function IsInvalidAccessID( $accessid )
	{
		$is_invalid   = true;
		$api_hostname = self::GenerateAPIHostname( $_SERVER["SERVER_NAME"] );
		$context      = self::CreateStreamContext( $api_hostname, "Aid=$accessid" );

		error_log( "Authenticating using: " . $api_hostname );

		if ( $json = file_get_contents( $api_hostname . "/auth/access/?Aid=$accessid", false, $context ) )
		{
			$obj = json_decode( $json );

			if ( "OK" == $obj->status )
			{
				if ( 1 == count( $obj->results ) )
				{
					if ( "PERMITTED" == $obj->results[0]->access )
					{
						$is_invalid = false;
					}
				}
			}
		}

		if ( $is_invalid ) error_log( $json );

		return $is_invalid;
	}

	static function GenerateAPIHostname( $hostname )
	{
		$port         = (-1 === strpos( $hostname, ".local" )) ? "443" : "8443";
		$api_hostname = "api-" . $hostname . ":" . $port;

		return ("" !== $_SERVER["HTTPS"]) ? "https://" . $api_hostname : "http://" . $api_hostname;
	}	

	/*
	 *  Copied from BaseSchema Process Places script with permission.
	 *  Copyright 2018 Daniel Bradley
	 */
	function CreateStreamContext( $hostname, $content )
	{
	    $verify_server = (-1 === strpos( $hostname, ".local" ));

	    $NL = "\r\n";

	    $content_type   = "Content-type: application/x-www-form-urlencoded" . $NL;
	    $content_length = "Content-Length: " . strlen( $content )           . $NL;

	    return stream_context_create
	    (
	        array
	        (
	            "http" => array
	            (
	                "method"        => "GET",
	                "header"        => $content_type . $content_length,
	                "content"       => $content,
	                "ignore_errors" => true,
	                "timeout"       => (float)30.0,
	            ),

	            "ssl" => array
	            (
	                "allow_self_signed" => !$verify_server,
	                "verify_peer"       =>  $verify_server,
	                "verify_peer_name"  =>  $verify_server,
	            ),
	        )
	    );
	}
}
